@@ -87,13 +87,20 @@
   (print-args o w)
   (.write w ")"))
 
-(defmethod print-method Object [o, ^Writer w]
+(defn- print-object [o, ^Writer w]
+  (when (instance? clojure.lang.IMeta o)
+    (print-meta o w))
   (.write w "#<")
-  (.write w (.getSimpleName (class o)))
-  (.write w " ")
+  (let [name (.getSimpleName (class o))]
+    (when (seq name) ;; anonymous classes have a simple name of ""
+      (.write w name)
+      (.write w " ")))
   (.write w (str o))
   (.write w ">"))
 
+(defmethod print-method Object [o, ^Writer w]
+  (print-object o w))
+
 (defmethod print-method clojure.lang.Keyword [o, ^Writer w]
   (.write w (str o)))
 
@@ -214,6 +221,40 @@
   (print-map m print-dup w)
   (.write w ")"))
 
+;; java.util
+(prefer-method print-method clojure.lang.IPersistentCollection java.util.Collection)
+(prefer-method print-method clojure.lang.IPersistentCollection java.util.RandomAccess)
+(prefer-method print-method java.util.RandomAccess java.util.List)
+(prefer-method print-method clojure.lang.IPersistentCollection java.util.Map)
+
+(defmethod print-method java.util.List [c, ^Writer w]
+  (if *print-readably*
+    (do
+      (print-meta c w)
+      (print-sequential "(" pr-on " " ")" c w))
+    (print-object c w)))
+
+(defmethod print-method java.util.RandomAccess [v, ^Writer w]
+  (if *print-readably*
+    (do
+      (print-meta v w)
+      (print-sequential "[" pr-on " " "]" v w))
+    (print-object v w)))
+
+(defmethod print-method java.util.Map [m, ^Writer w]
+  (if *print-readably*
+    (do
+      (print-meta m w)
+      (print-map m pr-on w))
+    (print-object m w)))
+
+(defmethod print-method java.util.Set [s, ^Writer w]
+  (if *print-readably*
+    (do
+      (print-meta s w)
+      (print-sequential "#{" pr-on " " "}" (seq s) w))
+    (print-object s w)))
+
 ;; Records
 
 (defmethod print-method clojure.lang.IRecord [r, ^Writer w]
@@ -241,7 +282,7 @@
   (print-meta s w)
   (print-sequential "#{" pr-on " " "}" (seq s) w))
 
-(def ^{:tag String 
+(def ^{:tag String
        :doc "Returns name string for char or nil if none"
        :added "1.0"} 
  char-name-string
@@ -343,7 +384,8 @@
                                      (agent-error o))
                               " FAILED"
                               ""))
-                    pr-on, "", ">", (list (if (and (instance? clojure.lang.IPending o) (not (.isRealized o)))
+                    pr-on, "", ">", (list (if (and (instance? clojure.lang.IPending o)
+                                                   (not (.isRealized ^clojure.lang.IPending o)))
                                             :pending
                                             @o)), w))
 
