@@ -543,13 +543,15 @@ Note this should only be used for the last one in the sequence"
   "Produce string parts for the mantissa (normalized 1-9) and exponent"
   [^Object f]
   (let [^String s (.toLowerCase (.toString f))
-        exploc (.indexOf s (int \e))]
+        exploc (.indexOf s (int \e))
+        dotloc (.indexOf s (int \.))]
     (if (neg? exploc)
-      (let [dotloc (.indexOf s (int \.))]
-        (if (neg? dotloc)
-          [s (str (dec (count s)))]
-          [(str (subs s 0 dotloc) (subs s (inc dotloc))) (str (dec dotloc))]))
-      [(str (subs s 0 1) (subs s 2 exploc)) (subs s (inc exploc))])))
+      (if (neg? dotloc)
+        [s (str (dec (count s)))]
+        [(str (subs s 0 dotloc) (subs s (inc dotloc))) (str (dec dotloc))])
+      (if (neg? dotloc)
+        [(subs s 0 exploc) (subs s (inc exploc))]
+        [(str (subs s 0 1) (subs s 2 exploc)) (subs s (inc exploc))]))))
 
 
 (defn- float-parts
@@ -564,14 +566,43 @@ Note this should only be used for the last one in the sequence"
       ["0" 0]
       [m2 (- (Integer/valueOf e) delta)])))
 
+(defn- ^String inc-s
+  "Assumption: The input string consists of one or more decimal digits,
+and no other characters.  Return a string containing one or more
+decimal digits containing a decimal number one larger than the input
+string.  The output string will always be the same length as the input
+string, or one character longer."
+  [^String s]
+  (let [len-1 (dec (count s))]
+    (loop [i (int len-1)]
+      (cond
+       (neg? i) (apply str "1" (repeat (inc len-1) "0"))
+       (= \9 (.charAt s i)) (recur (dec i))
+       :else (apply str (subs s 0 i)
+                    (char (inc (int (.charAt s i))))
+                    (repeat (- len-1 i) "0"))))))
+
 (defn- round-str [m e d w]
   (if (or d w)
     (let [len (count m)
-          round-pos (if d (+ e d 1))
-          round-pos (if (and w (< (inc e) (dec w)) 
-                             (or (nil? round-pos) (< (dec w) round-pos)))
-                      (dec w)
-                      round-pos)
+          ;; Every formatted floating point number should include at
+          ;; least one decimal digit and a decimal point.
+          w (if w (max 2 w))
+          round-pos (cond
+                     ;; If d was given, that forces the rounding
+                     ;; position, regardless of any width that may
+                     ;; have been specified.
+                     d (+ e d 1)
+                     ;; Otherwise w was specified, so pick round-pos
+                     ;; based upon that.
+                     ;; If e>=0, then abs value of number is >= 1.0,
+                     ;; and e+1 is number of decimal digits before the
+                     ;; decimal point when the number is written
+                     ;; without scientific notation.  Never round the
+                     ;; number before the decimal point.
+                     (>= e 0) (max (inc e) (dec w))
+                     ;; e < 0, so number abs value < 1.0
+                     :else (+ w e))
           [m1 e1 round-pos len] (if (= round-pos 0) 
                                   [(str "0" m) (inc e) 1 (inc len)]
                                   [m e round-pos len])]
@@ -582,22 +613,23 @@ Note this should only be used for the last one in the sequence"
             (let [round-char (nth m1 round-pos)
                   ^String result (subs m1 0 round-pos)]
               (if (>= (int round-char) (int \5))
-                (let [result-val (Integer/valueOf result)
-                      leading-zeros (subs result 0 (min (prefix-count result \0) (- round-pos 1)))
-                      round-up-result (str leading-zeros
-                                           (String/valueOf (+ result-val 
-                                                              (if (neg? result-val) -1 1))))
+                (let [round-up-result (inc-s result)
                       expanded (> (count round-up-result) (count result))]
-                  [round-up-result e1 expanded])
+                  [(if expanded
+                     (subs round-up-result 0 (dec (count round-up-result)))
+                     round-up-result)
+                   e1 expanded])
                 [result e1 false]))
             [m e false]))
         [m e false]))
     [m e false]))
 
 (defn- expand-fixed [m e d]
-  (let [m1 (if (neg? e) (str (apply str (repeat (dec (- e)) \0)) m) m)
+  (let [[m1 e1] (if (neg? e)
+                  [(str (apply str (repeat (dec (- e)) \0)) m) -1]
+                  [m e])
         len (count m1)
-        target-len (if d (+ e d 1) (inc e))]
+        target-len (if d (+ e1 d 1) (inc e1))]
     (if (< len target-len) 
       (str m1 (apply str (repeat (- target-len len) \0))) 
       m1)))
@@ -620,6 +652,21 @@ Note this should only be used for the last one in the sequence"
     (str "." m)
     (str (subs m 0 k) "." (subs m k))))
 
+(defn- convert-ratio [x]
+  (if (ratio? x)
+    ;; Usually convert to a double, only resorting to the slower
+    ;; bigdec conversion if the result does not fit within the range
+    ;; of a double.
+    (let [d (double x)]
+      (if (== d 0.0)
+        (if (not= x 0)
+          (bigdec x)
+          d)
+        (if (or (== d Double/POSITIVE_INFINITY) (== d Double/NEGATIVE_INFINITY))
+          (bigdec x)
+          d)))
+    x))
+
 ;; the function to render ~F directives
 ;; TODO: support rationals. Back off to ~D/~A is the appropriate cases
 (defn- fixed-float [params navigator offsets]
@@ -627,6 +674,7 @@ Note this should only be used for the last one in the sequence"
         d (:d params)
         [arg navigator] (next-arg navigator)
         [sign abs] (if (neg? arg) ["-" (- arg)] ["+" arg])
+        abs (convert-ratio abs)
         [mantissa exp] (float-parts abs)
         scaled-exp (+ exp (:k params))
         add-sign (or (:at params) (neg? arg))
@@ -634,6 +682,13 @@ Note this should only be used for the last one in the sequence"
         [rounded-mantissa scaled-exp expanded] (round-str mantissa scaled-exp 
                                                           d (if w (- w (if add-sign 1 0))))
         fixed-repr (get-fixed rounded-mantissa (if expanded (inc scaled-exp) scaled-exp) d)
+        fixed-repr (if (and w d
+                            (>= d 1)
+                            (= (.charAt fixed-repr 0) \0)
+                            (= (.charAt fixed-repr 1) \.)
+                            (> (count fixed-repr) (- w (if add-sign 1 0))))
+                     (subs fixed-repr 1)  ; chop off leading 0
+                     fixed-repr)
         prepend-zero (= (first fixed-repr) \.)]
     (if w
       (let [len (count fixed-repr)
@@ -663,7 +718,8 @@ Note this should only be used for the last one in the sequence"
 ;; TODO: support rationals. Back off to ~D/~A is the appropriate cases
 ;; TODO: define ~E representation for Infinity
 (defn- exponential-float [params navigator offsets]
-  (let [[arg navigator] (next-arg navigator)]
+  (let [[arg navigator] (next-arg navigator)
+        arg (convert-ratio arg)]
     (loop [[mantissa exp] (float-parts (if (neg? arg) (- arg) arg))]
       (let [w (:w params)
             d (:d params)
@@ -737,6 +793,7 @@ Note this should only be used for the last one in the sequence"
 ;; TODO: refactor so that float-parts isn't called twice
 (defn- general-float [params navigator offsets]
   (let [[arg _] (next-arg navigator)
+        arg (convert-ratio arg)
         [mantissa exp] (float-parts (if (neg? arg) (- arg) arg))
         w (:w params)
         d (:d params)
@@ -1073,7 +1130,7 @@ Note this should only be used for the last one in the sequence"
              s)))))
 
 (defn- capitalize-word-writer
-  "Returns a proxy that wraps writer, captializing all words"
+  "Returns a proxy that wraps writer, capitalizing all words"
   [^java.io.Writer writer]
   (let [last-was-whitespace? (ref true)] 
     (proxy [java.io.Writer] []
@@ -1145,7 +1202,7 @@ Note this should only be used for the last one in the sequence"
 
 (defn get-pretty-writer 
   "Returns the java.io.Writer passed in wrapped in a pretty writer proxy, unless it's 
-already a pretty writer. Generally, it is unneccesary to call this function, since pprint,
+already a pretty writer. Generally, it is unnecessary to call this function, since pprint,
 write, and cl-format all call it if they need to. However if you want the state to be 
 preserved across calls, you will want to wrap them with this. 
 
