@@ -11,9 +11,8 @@
 package clojure.lang;
 
 import java.io.Serializable;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
+import java.util.concurrent.Callable;
 import java.util.concurrent.atomic.AtomicReference;
 
 /*
@@ -180,10 +179,32 @@ public Iterator iterator(){
 
 public Object kvreduce(IFn f, Object init){
     init = hasNull?f.invoke(init,null,nullValue):init;
-    if(root != null){
-        return root.kvreduce(f,init);
-    }
-    return init;
+	if(RT.isReduced(init))
+		return ((IDeref)init).deref();
+	if(root != null){
+		init = root.kvreduce(f,init);
+		if(RT.isReduced(init))
+			return ((IDeref)init).deref();
+		else
+			return init;
+	}
+	return init;
+}
+
+public Object fold(long n, final IFn combinef, final IFn reducef,
+                   IFn fjinvoke, final IFn fjtask, final IFn fjfork, final IFn fjjoin){
+	//we are ignoring n for now
+	Callable top = new Callable(){
+		public Object call() throws Exception{
+			Object ret = combinef.invoke();
+			if(root != null)
+				ret = combinef.invoke(ret, root.fold(combinef,reducef,fjtask,fjfork,fjjoin));
+			return hasNull?
+			       combinef.invoke(ret,reducef.invoke(combinef.invoke(),null,nullValue))
+			       :ret;
+		}
+	};
+	return fjinvoke.invoke(top);
 }
 
 public int count(){
@@ -287,7 +308,7 @@ static final class TransientHashMap extends ATransientMap {
 			else
 				return notFound;
 		if (root == null)
-			return null;
+			return notFound;
 		return root.find(0, hash(key), key, notFound);
 	}
 
@@ -322,6 +343,7 @@ static interface INode extends Serializable {
 
     public Object kvreduce(IFn f, Object init);
 
+	Object fold(IFn combinef, IFn reducef, IFn fjtask, IFn fjfork, IFn fjjoin);
 }
 
 final static class ArrayNode implements INode{
@@ -383,14 +405,62 @@ final static class ArrayNode implements INode{
 	}
 
     public Object kvreduce(IFn f, Object init){
-        for(INode node : array)
-            {
-            if(node != null)
+        for(INode node : array){
+            if(node != null){
                 init = node.kvreduce(f,init);
-            }
+	            if(RT.isReduced(init))
+		            return init;
+	            }
+	        }
         return init;
     }
 
+	public Object fold(final IFn combinef, final IFn reducef,
+	                   final IFn fjtask, final IFn fjfork, final IFn fjjoin){
+		List<Callable> tasks = new ArrayList();
+		for(final INode node : array){
+			if(node != null){
+				tasks.add(new Callable(){
+					public Object call() throws Exception{
+						return node.fold(combinef, reducef, fjtask, fjfork, fjjoin);
+					}
+				});
+				}
+			}
+
+		return foldTasks(tasks,combinef,fjtask,fjfork,fjjoin);
+		}
+
+	static public Object foldTasks(List<Callable> tasks, final IFn combinef,
+	                          final IFn fjtask, final IFn fjfork, final IFn fjjoin){
+
+		if(tasks.isEmpty())
+			return combinef.invoke();
+
+		if(tasks.size() == 1){
+			Object ret = null;
+			try
+				{
+				return tasks.get(0).call();
+				}
+			catch(Exception e)
+				{
+				throw Util.sneakyThrow(e);
+				}
+			}
+
+		List<Callable> t1 = tasks.subList(0,tasks.size()/2);
+		final List<Callable> t2 = tasks.subList(tasks.size()/2, tasks.size());
+
+		Object forked = fjfork.invoke(fjtask.invoke(new Callable() {
+			public Object call() throws Exception{
+				return foldTasks(t2,combinef,fjtask,fjfork,fjjoin);
+			}
+		}));
+
+		return combinef.invoke(foldTasks(t1,combinef,fjtask,fjfork,fjjoin),fjjoin.invoke(forked));
+	}
+
 
 	private ArrayNode ensureEditable(AtomicReference<Thread> edit){
 		if(this.edit == edit)
@@ -625,6 +695,9 @@ final static class BitmapIndexedNode implements INode{
          return NodeSeq.kvreduce(array,f,init);
     }
 
+	public Object fold(IFn combinef, IFn reducef, IFn fjtask, IFn fjfork, IFn fjjoin){
+		return NodeSeq.kvreduce(array, reducef, combinef.invoke());
+	}
 
 	private BitmapIndexedNode ensureEditable(AtomicReference<Thread> edit){
 		if(this.edit == edit)
@@ -767,10 +840,10 @@ final static class HashCollisionNode implements INode{
 					return this;
 				return new HashCollisionNode(null, hash, count, cloneAndSet(array, idx + 1, val));
 			}
-			Object[] newArray = new Object[array.length + 2];
-			System.arraycopy(array, 0, newArray, 0, array.length);
-			newArray[array.length] = key;
-			newArray[array.length + 1] = val;
+			Object[] newArray = new Object[2 * (count + 1)];
+			System.arraycopy(array, 0, newArray, 0, 2 * count);
+			newArray[2 * count] = key;
+			newArray[2 * count + 1] = val;
 			addedLeaf.val = addedLeaf;
 			return new HashCollisionNode(edit, hash, count + 1, newArray);
 		}
@@ -814,6 +887,10 @@ final static class HashCollisionNode implements INode{
          return NodeSeq.kvreduce(array,f,init);
     }
 
+	public Object fold(IFn combinef, IFn reducef, IFn fjtask, IFn fjfork, IFn fjjoin){
+		return NodeSeq.kvreduce(array, reducef, combinef.invoke());
+	}
+
 	public int findIndex(Object key){
 		for(int i = 0; i < 2*count; i+=2)
 			{
@@ -1013,21 +1090,21 @@ private static INode createNode(int shift, Object key1, Object val1, int key2has
 	int key1hash = hash(key1);
 	if(key1hash == key2hash)
 		return new HashCollisionNode(null, key1hash, 2, new Object[] {key1, val1, key2, val2});
-	Box _ = new Box(null);
+	Box addedLeaf = new Box(null);
 	AtomicReference<Thread> edit = new AtomicReference<Thread>();
 	return BitmapIndexedNode.EMPTY
-		.assoc(edit, shift, key1hash, key1, val1, _)
-		.assoc(edit, shift, key2hash, key2, val2, _);
+		.assoc(edit, shift, key1hash, key1, val1, addedLeaf)
+		.assoc(edit, shift, key2hash, key2, val2, addedLeaf);
 }
 
 private static INode createNode(AtomicReference<Thread> edit, int shift, Object key1, Object val1, int key2hash, Object key2, Object val2) {
 	int key1hash = hash(key1);
 	if(key1hash == key2hash)
 		return new HashCollisionNode(null, key1hash, 2, new Object[] {key1, val1, key2, val2});
-	Box _ = new Box(null);
+	Box addedLeaf = new Box(null);
 	return BitmapIndexedNode.EMPTY
-		.assoc(edit, shift, key1hash, key1, val1, _)
-		.assoc(edit, shift, key2hash, key2, val2, _);
+		.assoc(edit, shift, key1hash, key1, val1, addedLeaf)
+		.assoc(edit, shift, key2hash, key2, val2, addedLeaf);
 }
 
 private static int bitpos(int hash, int shift){
@@ -1058,6 +1135,8 @@ static final class NodeSeq extends ASeq {
                  if(node != null)
                      init = node.kvreduce(f,init);
                  }
+             if(RT.isReduced(init))
+	             return init;
              }
         return init;
     }
