@@ -10,7 +10,7 @@
                                        :y-only y-only
                                        :common common}))))))
 
-(deftest compare-reflect-and-asm
+#_(deftest compare-reflect-and-asm
   (let [cl (.getContextClassLoader (Thread/currentThread))
         asm-reflector (AsmReflector. cl)
         java-reflector (JavaReflector. cl)]
@@ -18,7 +18,8 @@
                         java.lang.Object
                         java.io.FileInputStream
                         clojure.lang.Compiler
-                        clojure.lang.PersistentVector]]
+                        clojure.lang.PersistentVector
+                        java.lang.SuppressWarnings]]
       (nodiff (type-reflect classname :reflector asm-reflector)
               (type-reflect classname :reflector java-reflector)))))
 
