@@ -204,7 +204,7 @@
       1.2 nil
       "abc" nil ))
 
-;Tests that the comparator is preservered
+;Tests that the comparator is preserved
 ;The first element should be the same in each set if preserved.
 (deftest test-empty-sorted
   (let [inv-compare (comp - compare)]
@@ -682,7 +682,11 @@
 
     (interleave [] [3 4]) ()
     (interleave [1 2] []) ()
-    (interleave [] []) () ))
+    (interleave [] []) ()
+
+    (interleave [1]) '(1)
+
+    (interleave) () ))
 
 
 (deftest test-zipmap
@@ -944,7 +948,9 @@
       (range -2 -2) ()
       (range -2 -5) ()
 
-      (range 3 9 0) ()
+      (take 3 (range 3 9 0)) '(3 3 3)
+      (take 3 (range 9 3 0)) '(9 9 9)
+      (range 0 0 0) ()
       (range 3 9 1) '(3 4 5 6 7 8)
       (range 3 9 2) '(3 5 7)
       (range 3 9 3) '(3 6)
@@ -1108,10 +1114,10 @@
        [1 2 3 4 5] `(1 2 3 4 5)
        ;maps
        [] {:a 1 :b 2}
-       [:a 1 :b 2] (seq {:a 1 :b 2})
+       [:a 1 :b 2] (sort-by key {:a 1 :b 2})
        [] {[:a :b] 1 :c 2}
-       [:a :b 1 :c 2] (seq {[:a :b] 1 :c 2})
-       [:a 1 2 :b 3] (seq {:a [1 2] :b 3})
+       [:a :b 1 :c 2] (sort-by val {[:a :b] 1 :c 2})
+       [:a 1 2 :b 3] (sort-by key {:a [1 2] :b 3})
        ;Strings
        [] "12345"
        [\1 \2 \3 \4 \5] (seq "12345")
